@@ -382,6 +382,51 @@ gkr_pk_index_new (GkrKeyring *keyring, GnomeKeyringAttributeList *defaults)
 	return index;
 }
 
+gboolean
+gkr_pk_index_allows_secrets (GkrPkIndex *index)
+{
+	if (!index)
+		return FALSE;
+
+	g_return_val_if_fail (GKR_IS_PK_INDEX (index), FALSE);
+	g_return_val_if_fail (GKR_IS_KEYRING (index->keyring), FALSE);
+
+	/* 
+	 * Secrets can be stored on:
+	 *  - Disk base indexes (memory based doesn't make sense,
+	 *    it's just confusing to the user. It's not really 'stored').
+	 *  - Encrypted indexes. It's not secure to store password 
+	 *    in clear text indexes.
+	 */
+	 
+	return index->keyring->location && 
+	       !gkr_keyring_is_insecure (index->keyring); 
+}
+
+GkrPkIndex*
+gkr_pk_index_open_session (GnomeKeyringAttributeList *defaults)
+{
+	GkrKeyring *session;
+
+	session = gkr_keyrings_get_session ();
+	g_return_val_if_fail (session, NULL);
+	
+	return gkr_pk_index_new (session, defaults);
+}
+
+GkrPkIndex*
+gkr_pk_index_open_login (GnomeKeyringAttributeList *defaults)
+{
+	GkrKeyring *login;
+
+	if (!gkr_keyring_login_unlock (NULL))
+		return NULL;
+	
+	login = gkr_keyrings_get_login ();
+	g_return_val_if_fail (login, NULL);
+	
+	return gkr_pk_index_new (login, defaults);
+}
 
 GkrPkIndex*
 gkr_pk_index_open (GQuark index_location, const gchar *name, 
@@ -459,7 +504,6 @@ gkr_pk_index_get_uint (GkrPkIndex *index, gkrconstid digest,
 	if (item != NULL)
 		attr = gkr_attribute_list_find (item->attributes, field);
 		
-	attr = gkr_attribute_list_find (item->attributes, field);
 	if (!attr) {
 		attr = find_default_attribute (index, field);
 		if (!attr)
@@ -650,6 +694,13 @@ gkr_pk_index_set_secret (GkrPkIndex *index, gkrconstid digest,
 		index = gkr_pk_index_default ();
 
 	g_return_val_if_fail (GKR_IS_PK_INDEX (index), FALSE);
+	
+	/* Cannot store secrets in an insecure keyring. Caller should have checked this. */
+	if (val != NULL && !gkr_pk_index_allows_secrets (index)) {
+		g_warning ("gkr_pk_index_set_secret() called on an insecure keyring. Cannot "
+		           "store secrets in a text based or otherwise insecure keyring.");
+		return FALSE;
+	}
 
 	item = find_item_for_digest (index, digest, TRUE);
 	if (!item)
