@@ -269,9 +269,6 @@ index_correspending_public_key (GkrSshStorage *storage, GQuark loc, gkrconstid d
 
 	g_return_if_fail (loc);
 	
-	index = gkr_ssh_storage_index (GKR_PK_STORAGE (storage), loc);
-	g_return_if_fail (index);
-	
 	ploc = public_location_for_private (loc);
 	g_return_if_fail (ploc);
 	
@@ -302,6 +299,7 @@ index_correspending_public_key (GkrSshStorage *storage, GQuark loc, gkrconstid d
 	}
 
 	/* Write key to the indexes */
+	index = gkr_ssh_storage_index (GKR_PK_STORAGE (storage), loc);
 	if (!gkr_pk_index_has_value (index, digest, GKR_PK_INDEX_PUBLIC_KEY)) {
 		data = gkr_pkix_der_write_public_key (sexp, &n_data);
 		g_return_if_fail (data != NULL);
@@ -330,6 +328,8 @@ parsed_pem_block (GQuark type, const guchar *data, gsize n_data,
 	gchar *comment;
 	gkrid digest;
 	
+g_message("saw PEM block: %s", g_quark_to_string (type));
+
 	/* Only handle SSHv2 private keys */
 	if (type != PEM_RSA_PRIVATE_KEY && type != PEM_DSA_PRIVATE_KEY)
 		return;
@@ -343,6 +343,7 @@ parsed_pem_block (GQuark type, const guchar *data, gsize n_data,
 	
 	/* If it's encrypted ... */
 	dekinfo = gkr_pkix_openssl_get_dekinfo (headers);
+g_message("dekinfo: %s", dekinfo);
 	if (dekinfo) {
 		/* This key was specifically requested to be loaded */
 		if (gkr_id_equals (digest, pv->specific_load_request)) {
@@ -351,6 +352,7 @@ parsed_pem_block (GQuark type, const guchar *data, gsize n_data,
 			
 		/* Nobody's asking us to load this key just yet */
 		} else {
+g_message("skipping loading of wrong key");
 			ctx->result = GKR_PKIX_SUCCESS;
 			sexp = NULL;
 		}
@@ -397,6 +399,7 @@ storage_load_private_key (GkrSshStorage *storage, GQuark loc, GError **err)
 	g_return_val_if_fail (loc, FALSE);
 	g_return_val_if_fail (!err || !*err, FALSE);
 	
+g_message("loading SSH private key at: %s", g_quark_to_string (loc));
 	if (!gkr_location_read_file (loc, &data, &n_data, err))
 		return FALSE;
 
@@ -411,7 +414,7 @@ storage_load_private_key (GkrSshStorage *storage, GQuark loc, GError **err)
 		return TRUE;
 	
 	if (ctx.result == GKR_PKIX_FAILURE) {
-		g_set_error (err, GKR_PK_STORAGE_ERROR, 0, _("Couldn't read secure shell key private key: %s"),
+		g_set_error (err, GKR_PK_STORAGE_ERROR, 0, _("Couldn't read secure shell private key: %s"),
 		             g_quark_to_string (loc));
 		return FALSE;
 	} else if (ctx.result == GKR_PKIX_UNRECOGNIZED) {
@@ -665,21 +668,17 @@ gkr_ssh_storage_index (GkrPkStorage *storage, GQuark unused)
 {
  	GkrSshStoragePrivate *pv = GKR_SSH_STORAGE_GET_PRIVATE (storage);
  	GnomeKeyringAttributeList *attrs;
-	GQuark kloc;
 	
 	if (!pv->index) {
-		/* We default to a keyring stored on the computer */
-		kloc = gkr_location_from_child (GKR_LOCATION_VOLUME_LOCAL, 
-		                                "pk-storage.keyring");
-		
 		/* Default attributes for our index */
 		attrs = gnome_keyring_attribute_list_new ();
 		gnome_keyring_attribute_list_append_string (attrs, "purposes", "ssh-authentication");
 		
-		pv->index = gkr_pk_index_open (kloc, "pk-storage", attrs);
-		gnome_keyring_attribute_list_free (attrs);
+		pv->index = gkr_pk_index_open_login (attrs);
+		if (!pv->index)
+			pv->index = gkr_pk_index_open_session (attrs);
 		
-		g_return_val_if_fail (pv->index, NULL);
+		gnome_keyring_attribute_list_free (attrs);
 	}
 	
 	return pv->index;
